@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -19,12 +19,32 @@
  *
  */
 
+enum at_util_sms_store {
+	AT_UTIL_SMS_STORE_SM =	0,
+	AT_UTIL_SMS_STORE_ME =	1,
+	AT_UTIL_SMS_STORE_MT =	2,
+	AT_UTIL_SMS_STORE_SR =	3,
+	AT_UTIL_SMS_STORE_BM =	4,
+};
+
 void decode_at_error(struct ofono_error *error, const char *final);
-void dump_response(const char *func, gboolean ok, GAtResult *result);
 gint at_util_call_compare_by_status(gconstpointer a, gconstpointer b);
 gint at_util_call_compare_by_phone_number(gconstpointer a, gconstpointer b);
+gint at_util_call_compare_by_id(gconstpointer a, gconstpointer b);
 gint at_util_call_compare(gconstpointer a, gconstpointer b);
 GSList *at_util_parse_clcc(GAtResult *result);
+gboolean at_util_parse_reg(GAtResult *result, const char *prefix,
+				int *mode, int *status,
+				int *lac, int *ci, int *tech,
+				unsigned int vendor);
+gboolean at_util_parse_reg_unsolicited(GAtResult *result, const char *prefix,
+					int *status, int *lac,
+					int *ci, int *tech,
+					unsigned int vendor);
+
+gboolean at_util_parse_sms_index_delivery(GAtResult *result, const char *prefix,
+						enum at_util_sms_store *store,
+						int *index);
 
 struct cb_data {
 	void *cb;
@@ -47,6 +67,18 @@ static inline struct cb_data *cb_data_new(void *cb, void *data)
 	return ret;
 }
 
+static inline int at_util_convert_signal_strength(int strength)
+{
+	int result;
+
+	if (strength == 99)
+		result = -1;
+	else
+		result = (strength * 100) / 31;
+
+	return result;
+}
+
 #define DECLARE_FAILURE(e) 			\
 	struct ofono_error e;			\
 	e.type = OFONO_ERROR_TYPE_FAILURE;	\
